import traceback
import logging
import requests
import sys

ch = logging.StreamHandler()
ch.setLevel(logging.ERROR)
dl = logging.getLogger(__name__)
dl.addHandler(ch)


def merge(x, y):
    z = x.copy()
    z.update(y)
    return z


CONF = {
    'baseurl': 'https://localhost:6500',
    'token': None,
    'debug': False,
    'prefix': '',
    'set_mode': 'raw',
    'headers': {},
    'logger': dl,
    'verify': True,  # this can be False or a cacert path
    'timeout': 15,
    'cert': None  # pem or ('/path/client.cert', '/path/client.key')
}


class kvdn_client:
    kvdn_py_version = '2.0.3'

    def __init__(self, **kwargs):
        for key, value in kwargs.iteritems():
            CONF[key] = value
        if CONF['token'] is not None:
            CONF['headers'].update({'Authorization': 'Bearer %s' % CONF['token']})
        self.session = requests.Session()
        self.SVER = self.session.get(CONF['baseurl'] + CONF['prefix'] + '/__VERSION', verify=CONF['verify'],
                                     timeout=CONF['timeout'], cert=CONF['cert'], headers=CONF['headers'])


    def set(self, straddr, key, data, **kwargs):
        if CONF['set_mode'] == 'raw':
            return self.setRaw(straddr, key, data, **kwargs)
        elif CONF['set_mode'] == 'json':
            return self.setJson(straddr, key, data, **kwargs)

    def version(self):
        return self.SVER.text

    def get(self, straddr, key, **kwargs):
        resp, content = kvdn_req(self.session, CONF['baseurl'] + CONF['prefix'] + '/X/' + straddr + '/' + key,
                                 headers=CONF['headers'])
        return content

    def setRaw(self, straddr, key, data, type='text/plain', **kwargs):
        resp, content = kvdn_req(self.session, CONF['baseurl'] + CONF['prefix'] + '/R/' + straddr + '/' + key,
                                 method='PUT', data=data, headers=CONF['headers'].update({'Content-Type': type}))
        return content

    def setJson(self, straddr, key, data, type='application/json', **kwargs):
        resp, content = kvdn_req(self.session, CONF['baseurl'] + CONF['prefix'] + '/X/' + straddr + '/' + key,
                                 method='PUT', data=data, headers=CONF['headers'].update({'Content-Type': type}))
        return content

    def getKeys(self, straddr, **kwargs):
        resp, content = kvdn_req(self.session, CONF['baseurl'] + CONF['prefix'] + '/KEYS/' + straddr + '/',
                                 headers=CONF['headers'])
        return content

    def delete(self, straddr, key, **kwargs):
        resp, content = kvdn_req(self.session, CONF['baseurl'] + CONF['prefix'] + '/X/' + straddr + '/' + key,
                                 method='DELETE', data='', headers=CONF['headers'])
        return content

    def submit_cas(self, straddr, data, type='text/plain', **kwargs):
        resp, content = kvdn_req(self.session, CONF['baseurl'] + CONF['prefix'] + '/X/' + straddr, method='POST',
                                 data=data, headers=CONF['headers'].update({'Content-Type': type}))
        return content  # the returned content should be the hash of data as a key

    def submit_uuid(self, straddr, data, type='text/plain', **kwargs):
        resp, content = kvdn_req(self.session, CONF['baseurl'] + CONF['prefix'] + '/U/' + straddr, method='POST',
                                 data=data, headers=CONF['headers'].update({'Content-Type': type}))
        return content  # the returned content should be a uuid key


def kvdn_req(session, url, method=None, data=None, **kwargs):
    if CONF['debug'] is True:
        import pprint
        pprint.PrettyPrinter(indent=4).pprint(kwargs)
        pprint.PrettyPrinter(indent=4).pprint(CONF)
    if method is None:
        method = 'GET'
    req = requests.Request(method, url, data=data, headers=CONF['headers'])
    try:
        p = req.prepare()
        s = session
        resp = s.send(p, verify=CONF['verify'], timeout=CONF['timeout'], cert=CONF['cert'])
        return resp, resp.text
    except:
        CONF['logger'].error("could not make kvdn request " + traceback.format_exc())
        return "", ""



from kvdn_client import kvdn_client
import fileinput
import argparse
import sys, os
import json


def main(args=None):
    parser = argparse.ArgumentParser(description='KVDN PYCLIENT')
    parser.add_argument('--verify', type=str, default=True, help='True, False, or path to ca')
    parser.add_argument('--noverify', action='store_true', help='do not verify host certificate')
    parser.add_argument('--version', action='store_true', default=False, help='get server version')
    parser.add_argument('--cert', type=str, default='', help='pa./th to certificate')
    parser.add_argument('--type', type=str, default='text/plain', help='MIME type')
    parser.add_argument('--debug', action='store_true', help='debug')
    parser.add_argument('--set', action='store_true', help='set a value', )
    parser.add_argument('--submit', action='store_true', help='submit a value', )
    parser.add_argument('--delete', action='store_true', help='delete a value', )
    parser.add_argument('straddr', type=str, help='the kvdn map address to work with')
    parser.add_argument('--key', type=str, default='', help='the kvdn key to work with')

    s = ''
    args = parser.parse_args()
    if (args.set or args.submit):
        s = sys.stdin.read()
    _baseurl = ''
    _token = ''
    krgs = {}
    varargs = vars(args)
    if varargs['verify'] is '':
        varargs['verify'] = True

    try:
        _baseurl = os.environ["KVDN_BASE_URL"]
    except KeyError:
        _baseurl = 'http://localhost:6500'
        sys.stderr.write("using default url : " + _baseurl + '\n')

    try:
        _token = os.environ["CACERT"]
        krgs["verify"] = _token
    except Exception:
        pass
    try:
        _token = os.environ["CERT"]
        try:
            _token = json.load(_token)
            krgs["verify"] = _token
        except Exception:
            krgs["cert"] = _token
    except Exception:
        pass

    try:
        _token = os.environ["JWT_TOKEN"]
        k = kvdn_client(baseurl=_baseurl, token=_token, **varargs)
    except KeyError:
        sys.stderr.write('JWT_TOKEN not set \n')
        k = kvdn_client(baseurl=_baseurl)

    if (args.debug):
        sys.stderr.write("KVDN VERSION: " + k.version() + "\n")

    if (args.set):
        print k.set(args.straddr, args.key, s)
    elif(args.version):
        print k.version()
    elif (args.submit):
        print k.submit_cas(args.straddr, s)
    else:
        if (args.delete):
            print k.delete(args.straddr, args.key)
        elif (args.key):
            print k.get(args.straddr, args.key)
        else:
            print k.getKeys(args.straddr)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

try:
    long_description = open("README.rst").read()
except IOError:
    long_description = ""

setup(
    name="kvdnc",

    version="2.0.4",
    description="kvdn client library and tool",
    license="Apache",
    author="Grant Haywood",
    packages=["kvdn_client"],
    install_requires=["requests"],
    long_description=long_description,
      entry_points={
          'console_scripts': [
              'kvdn-cli = kvdn_client.__main__:main'
          ]
      },
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
    ]
)
